﻿ArchimondeShackledTormentHelperDB = {};

local CTimerAfter = C_Timer.After;
local tableNames = {};
local LocalPlayerFullName = UnitName("player").." - "..GetRealmName();
local MainFrame = CreateFrame("Frame", "ArchimondeShackledTormentHelperFrame", UIParent);

local function InitializeGUI_DB()
	if (ArchimondeShackledTormentHelperDB[LocalPlayerFullName] == nil) then
		ArchimondeShackledTormentHelperDB[LocalPlayerFullName] = { };
	end
	local defaults = {
		PositionX = 0,
		PositionY = 0,
		Point = "CENTER",
		RelativePoint = "CENTER",
	};
	for key, value in pairs(defaults) do
		if (ArchimondeShackledTormentHelperDB[LocalPlayerFullName][key] == nil) then
			ArchimondeShackledTormentHelperDB[LocalPlayerFullName][key] = value;
		end
	end
	-- // delete unused keys
	ArchimondeShackledTormentHelperDB[LocalPlayerFullName].RelativeTo = nil;
	-- //
	if (not MainFrame.lines) then
		local db = ArchimondeShackledTormentHelperDB[LocalPlayerFullName];
		MainFrame:SetHeight(100);
		MainFrame:SetWidth(150);
		MainFrame:SetPoint(db.Point, nil, db.RelativePoint, db.PositionX, db.PositionY);
		MainFrame:SetBackdrop({
			bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = 1,
			tileSize = 16,
			edgeSize = 16,
			insets = { left = 3, right = 3, top = 3, bottom = 3 } 
		});
		MainFrame:SetBackdropColor(0.1, 0.1, 0.1, 1);
		MainFrame:SetBackdropBorderColor(0.1, 0.1, 0.1, 1);
		MainFrame:EnableMouse(1);
		MainFrame:SetMovable(1);
		MainFrame:SetFrameStrata("DIALOG");
		MainFrame:SetToplevel(1);
		MainFrame:SetClampedToScreen(1);
		MainFrame:SetScript("OnMouseDown", function() MainFrame:StartMoving(); end);
		MainFrame:SetScript("OnMouseUp", function()
			MainFrame:StopMovingOrSizing();
			local point, relativeTo, relativePoint, xOffset, yOffset = MainFrame:GetPoint(1);
			ArchimondeShackledTormentHelperDB[LocalPlayerFullName] = {
				PositionX = xOffset,
				PositionY = yOffset,
				Point = point,
				RelativePoint = relativePoint,
			};
		end);
		MainFrame:Hide();
		
		MainFrame.lines = {};
		for i = 1, 10 do
			local line = MainFrame:CreateFontString("ArchimondeShackledTormentHelperFrame_Line"..tostring(i), "ARTWORK", "GameFontHighlight");
			line:SetFont(GameFontNormal:GetFont(), 12);
			line:SetPoint("TOPLEFT", MainFrame, "TOPLEFT", 10, -12 * i);
			line:SetText("");
			tinsert(MainFrame.lines, line);
		end
	end
end

local function UpdateGUI()
	local _tNames = {};
	for _, value in pairs(tableNames) do
		_tNames[#_tNames + 1] = value;
	end
	for i = 1, 10 do
		local line = MainFrame.lines[i];
		if (_tNames[i] ~= nil) then
			local color = RAID_CLASS_COLORS[select(2, UnitClass(_tNames[i])) or "WARRIOR"];
			line:SetTextColor(color.r, color.g, color.b, 1);
			line:SetText(_tNames[i]);
		else
			line:SetText("");
		end
	end
end

local function PLAYER_ENTERING_WORLD()
	MainFrame:UnregisterEvent("PLAYER_ENTERING_WORLD");
	InitializeGUI_DB();
end

local function ENCOUNTER_START(...)
	local encounterID = ...;
	if (encounterID == 1799) then
		wipe(tableNames);
		MainFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		MainFrame:Show();
		UpdateGUI();
		print("Archimonde helper is loaded");
	end
end

local function ENCOUNTER_END()
	MainFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	MainFrame:Hide();
end

local function COMBAT_LOG_EVENT_UNFILTERED(...)
	local _, eventType, _, _, _, _, _, _, destName, _, _, spellID = ...;
	if (eventType == "SPELL_AURA_APPLIED") then
		if (spellID == 184964) then
			tableNames[#tableNames+1] = destName;
			table.sort(tableNames);
			UpdateGUI();
		end
	elseif (eventType == "SPELL_AURA_REMOVED") then
		if (spellID == 184964) then
			for index, value in pairs(tableNames) do
				if (value == destName) then
					tableNames[index] = nil;
				end
			end
			UpdateGUI();
		end
	end
end

MainFrame:RegisterEvent("ENCOUNTER_START");
MainFrame:RegisterEvent("ENCOUNTER_END");
MainFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
MainFrame:SetScript("OnEvent", function(this, event, ...)
	if (event == "ENCOUNTER_START") then
		ENCOUNTER_START(...);
	elseif (event == "ENCOUNTER_END") then
		ENCOUNTER_END();
	elseif (event == "PLAYER_ENTERING_WORLD") then
		PLAYER_ENTERING_WORLD();
	else
		COMBAT_LOG_EVENT_UNFILTERED(...);
	end
end);

SLASH_ASTH1 = '/asth';
SlashCmdList["ASTH"] = function(msg, editBox)
	ENCOUNTER_START(1799);
	local testNames = { "Doctor Gideon", "Arzt Gideon", "Барон Гедеон", "Dottore Gideon", "Médico Gedeón" };
	for i = 1, 5 do
		COMBAT_LOG_EVENT_UNFILTERED(nil, "SPELL_AURA_APPLIED", nil, nil, nil, nil, nil, nil, testNames[i], nil, nil, 184964);
	end
	CTimerAfter(2.0, function()
		for i = 1, 5 do
			CTimerAfter(i, function()
				COMBAT_LOG_EVENT_UNFILTERED(nil, "SPELL_AURA_REMOVED", nil, nil, nil, nil, nil, nil, testNames[6-i], nil, nil, 184964);
			end);
		end
	end);
	CTimerAfter(10.0, ENCOUNTER_END);
end
